@@ -3,7 +3,7 @@
 ;; Copyright (C) 2006-2009, 2011-2012, 2015-2018
 ;;   Phil Hagelberg, Doug Alcorn, Will Farrington, Chen Bin
 ;;
-;; Version: 6.0.0
+;; Version: 6.0.1
 ;; Author: Phil Hagelberg, Doug Alcorn, and Will Farrington
 ;; Maintainer: Chen Bin <chenbin.sh@gmail.com>
 ;; URL: https://github.com/technomancy/find-file-in-project
@@ -32,11 +32,22 @@
 ;;; Commentary:
 
 ;; This program provides methods to find file in project.
+;;
+;; Features,
 ;; - Only dependency is BSD/GNU find
 ;; - Works on Windows with minimum setup
 ;; - Works on Tramp Mode (https://www.emacswiki.org/emacs/TrampMode)
 ;; - fd (faster alternative of find, see https://github.com/sharkdp/fd) is supported
-;; - Uses only native API `completing-read' and supports ido/ivy/helm out of box
+;; - Uses native API `completing-read' and supports ido/helm/ivy/consult/selectrum out of box.
+;;
+;;   Ido setup,
+;;     (ido-mode 1)
+;;
+;;   Helm setup,
+;;     (helm-mode 1)
+;;
+;;   Ivy setup,
+;;     (ivy-mode 1)
 ;;
 ;; Usage,
 ;;   - You can insert "(setq ffip-use-rust-fd t)" into ".emacs" to use fd (alternative of find)
@@ -147,20 +158,39 @@
 (require 'windmove)
 (require 'subr-x)
 
-(defvar ffip-use-rust-fd nil "Use rust fd instead of find.")
+(defgroup ffip nil
+  "Find File in Project."
+  :group 'convenience)
 
-(defvar ffip-rust-fd-respect-ignore-files t
-  "Don 't show search results from '.*ignore' files.")
+(defcustom ffip-use-rust-fd nil
+  "Use rust fd instead of find."
+  :link '(url-link :tag "fd @ GitHub"
+                   "https://github.com/sharkdp/fd")
+  :group 'ffip
+  :type 'boolean
+  :safe #'booleanp)
 
-(defvar ffip-rust-fd-extra-opts ""
-  "Rust fd extra options passed to cli.")
+
+(defcustom ffip-rust-fd-respect-ignore-files t
+  "Don't show search results from '.*ignore' files."
+  :group 'ffip
+  :type 'boolean
+  :safe #'booleanp)
+
+
+(defcustom ffip-rust-fd-extra-opts ""
+  "Rust fd extra options passed to cli."
+  :group 'ffip
+  :type 'string)
 
 (defvar ffip-filename-history nil)
 
-(defvar ffip-strip-file-name-regex
+(defcustom ffip-strip-file-name-regex
   "\\(\\.mock\\|\\.test\\|\\.mockup\\)"
   "Strip file name to get minimum keyword with this regex.
-It's used by `find-file-with-similar-name'.")
+It's used by `find-file-with-similar-name'."
+  :group 'ffip
+  :type 'regexp)
 
 (defvar ffip-diff-find-file-before-hook nil
   "Hook before `ffip-diff-find-file' move focus out of *ffip-diff* buffer.")
@@ -197,7 +227,7 @@ The file path is passed to the hook as the first argument.")
 (defun ffip-git-diff-current-file ()
   "Run 'git diff version:current-file current-file'."
   (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (completing-read "diff current file:" (ffip-diff-git-versions))))
+         (line (completing-read "diff current file: " (ffip-diff-git-versions))))
     (ffip-shell-command-to-string (format "git --no-pager diff %s:%s %s"
                                      (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))
                                      (file-relative-name buffer-file-name default-directory)
@@ -206,7 +236,7 @@ The file path is passed to the hook as the first argument.")
 (defun ffip-git-diff-project()
   "Run 'git diff version' in project."
   (let* ((default-directory (locate-dominating-file default-directory ".git"))
-         (line (completing-read "diff current file:" (ffip-diff-git-versions)))
+         (line (completing-read "diff current file: " (ffip-diff-git-versions)))
          (version (replace-regexp-in-string "^ *\\*? *" "" (car (split-string line "|" t)))))
     (ffip-shell-command-to-string (format "git --no-pager diff %s" version))))
 
@@ -221,7 +251,7 @@ The file path is passed to the hook as the first argument.")
     ("`git log -p` current file" . (ffip-shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git --no-pager log --date=short -p '%s'"
                                                      (buffer-file-name))))
     ("`git log -S keyword -p` in project" . (ffip-shell-command-to-string (format "cd $(git rev-parse --show-toplevel) && git --no-pager log --date=short -S'%s' -p"
-                                                              (read-string "Git search string:"))))
+                                                              (read-string "Git search string: "))))
     ("Diff from `kill-ring'" . (car kill-ring)))
   "The list of back-ends.
 If back-end is string, it's run in `shell-command-to-string'.
@@ -229,14 +259,21 @@ If it's a function or expression, it'll be executed and return a string.
 
 The output is inserted into *ffip-diff* buffer.")
 
-(defvar ffip-find-executable nil "Path of GNU find.  If nil we will guess.")
+(defcustom ffip-find-executable nil
+  "Path of GNU find.  If nil we will guess."
+  :group 'ffip
+  :type 'string)
 
-(defvar ffip-project-file '(".svn" ".hg" ".git")
+(defcustom ffip-project-file '(".svn" ".hg" ".git")
   "The file/directory used to locate project root.
-May be set using .dir-locals.el.  Checks each entry if set to a list.")
+May be set using .dir-locals.el.  Checks each entry if set to a list."
+  :group 'ffip
+  :type '(repeat string))
 
-(defvar ffip-patterns nil
-  "List of glob patterns to look for with `find-file-in-project'.")
+(defcustom ffip-patterns nil
+  "List of glob patterns to look for with `find-file-in-project'."
+  :group 'ffip
+  :type '(repeat string))
 
 (defvar ffip-match-path-instead-of-filename nil
   "Match full path instead of file name.")
@@ -245,8 +282,8 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
 ;;
 ;; But for "rust fd", only "test/*" matches "./test/" and "./dir/test/";
 ;; "*/test/*" won't match "./test/" but matches "./dir/test/"
-;; Maybe it's bug of fd.
-(defvar ffip-prune-patterns
+;; Maybe it's a fd bug.
+(defcustom ffip-prune-patterns
   '(;; VCS
     "*/.git"
     "*/.svn"
@@ -263,9 +300,11 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
     "*/bower_components"
     "*/.gradle"
     "*/.cask")
-  "Ignored directories(prune patterns).")
+  "Ignored directories(prune patterns)."
+  :group 'ffip
+  :type '(repeat string))
 
-(defvar ffip-ignore-filenames
+(defcustom ffip-ignore-filenames
   '(;; VCS
     ;; project misc
     "*.log"
@@ -329,25 +368,35 @@ May be set using .dir-locals.el.  Checks each entry if set to a list.")
     "*.elc"
     ;; Python
     "*.pyc")
-  "Ignore file names.  Wildcast is supported.")
+  "Ignored file names.  Wildcast is supported."
+  :group 'ffip
+  :type '(repeat string))
 
-(defvar ffip-find-options ""
+(defcustom ffip-find-options ""
   "Extra options to pass to `find' when using `find-file-in-project'.
 
-Use this to exclude portions of your project: \"-not -regex \\\".*svn.*\\\"\".")
+Use this to exclude portions of your project: \"-not -regex \\\".*svn.*\\\"\"."
+  :group 'ffip
+  :type 'string)
 
-(defvar ffip-find-pre-path-options ""
+(defcustom ffip-find-pre-path-options ""
   "Options for find program.
 
 GNU Find requires '-H', '-L', '-P', '-D' and `-O' appear before first path '.'.
-For example, use '-L' to follow symbolic links.")
+For example, use '-L' to follow symbolic links."
+  :group 'ffip
+  :type 'string)
 
-(defvar ffip-project-root nil
-  "If non-nil, overrides the project root directory location.")
+(defcustom ffip-project-root nil
+  "If non-nil, overrides the project root directory location."
+  :group 'ffip
+  :type 'string)
 
-(defvar ffip-project-root-function nil
+(defcustom ffip-project-root-function nil
   "If non-nil, this function is called to determine the project root.
-This overrides variable `ffip-project-root' when set.")
+This overrides variable `ffip-project-root' when set."
+  :group 'ffip
+  :type 'function)
 
 (defvar ffip-debug nil "Print debug information.")
 
@@ -372,8 +421,10 @@ This overrides variable `ffip-project-root' when set.")
   (message "%s => kill-ring" p))
 
 ;;;###autoload
-(defvar ffip-find-relative-path-callback 'ffip-copy-without-change
-  "The callback after calling `find-relative-path'.")
+(defcustom ffip-find-relative-path-callback 'ffip-copy-without-change
+  "The callback after calling `find-relative-path'."
+  :group 'ffip
+  :type 'function)
 
 (defun ffip--some (predicate seq)
   "Return if PREDICATE is t for any element of SEQ."
@@ -571,7 +622,7 @@ ACTION is a lambda function to call after selecting a result.
 This function returns the selected candidate or nil."
   (let* (selected)
     (cond
-     ((and action (= 1 (length collection)))
+     ((= 1 (length collection))
       ;; select the only candidate immediately
       (setq selected (car collection)))
 
@@ -581,8 +632,9 @@ This function returns the selected candidate or nil."
 
     (when selected
       ;; make sure only the string/file is passed to action
-      (let* ((default-directory (ffip-get-project-root-directory)))
-        (funcall action (if (consp selected) (cdr selected) selected))))))
+      (let* ((default-directory (ffip-get-project-root-directory))
+             (result (if (consp selected) (cdr selected) selected)))
+        (if action (funcall action result) result)))))
 
 (defun ffip-create-shell-command (keyword find-directory-p)
   "Produce command to search KEYWORD.
@@ -746,7 +798,7 @@ This function is the API to find files."
 You can move .dir-locals.el to root directory.
 See (info \"(Emacs) Directory Variables\") for details."
   (interactive)
-  (let* ((root (read-directory-name "Project root directory?" default-directory))
+  (let* ((root (read-directory-name "Project root directory: " default-directory))
          (file (if (and root (file-exists-p root))
                    (concat (file-name-as-directory root) ".dir-locals.el"))))
     (when file
@@ -811,7 +863,7 @@ You can override this by setting the variable `ffip-project-root'."
   (or (and (region-active-p) (ffip--read-selected))
       (thing-at-point 'filename)
       (thing-at-point 'symbol)
-      (read-string "No file name at point. Please provide one:")))
+      (read-string "No file name at point. Please provide one: ")))
 
 (defun ffip--guess-physical-path (file)
   "Return physical full path of FILE which does exist."
@@ -1197,7 +1249,7 @@ If NUM is not nil, the corresponding backend is executed directly."
                              (ffip-backend-description b)) t)
         (setq i (+ 1 i)))
       (ffip-completing-read
-       "Run diff backend:"
+       "Run diff backend: "
        descriptions
        (lambda (file)
          (if (string-match "^\\([0-9]+\\): " file)
